@@ -436,8 +436,13 @@ PianoReturn_t PianoResponse (PianoHandle_t *ph, PianoRequest_t *req) {
 			break;
 		}
 
+		case PIANO_REQUEST_ADD_TIRED_SONG: {
+			PianoSong_t * const song = req->data;
+			song->rating = PIANO_RATE_TIRED;
+			break;
+		}
+
 		case PIANO_REQUEST_ADD_SEED:
-		case PIANO_REQUEST_ADD_TIRED_SONG:
 		case PIANO_REQUEST_SET_QUICKMIX:
 		case PIANO_REQUEST_BOOKMARK_SONG:
 		case PIANO_REQUEST_BOOKMARK_ARTIST:
@@ -634,6 +639,11 @@ PianoReturn_t PianoResponse (PianoHandle_t *ph, PianoRequest_t *req) {
 						feedbackSong->rating = getBoolDefault (s, "isPositive",
 								false) ?  PIANO_RATE_LOVE : PIANO_RATE_BAN;
 
+						json_object *v;
+						feedbackSong->length =
+								json_object_object_get_ex (s, "trackLength", &v) ?
+								json_object_get_int (v) : 0;
+
 						info->feedback = PianoListAppendP (info->feedback,
 								feedbackSong);
 					}
