@@ -39,8 +39,10 @@ typedef struct {
 	/* first item is current song */
 	PianoSong_t *playlist;
 	PianoSong_t *songHistory;
-	PianoStation_t *curStation;
-	char doQuit;
+	/* station of current song and station used to fetch songs from if playlist
+	 * is empty */
+	PianoStation_t *curStation, *nextStation;
+	sig_atomic_t doQuit;
 	BarReadlineFds_t input;
 	unsigned int playerErrors;
 } BarApp_t;
