@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2008-2014
+Copyright (c) 2008-2018
 	Lars-Dominik Braun <lars@6xq.net>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -21,7 +21,16 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 */
 
-/* receive/play audio stream */
+/* receive/play audio stream.
+ *
+ * There are two threads involved here:
+ * BarPlayerThread
+ * 		Sets up the stream and fetches the data into a ffmpeg buffersrc
+ * BarAoPlayThread
+ * 		Reads data from the filter chain’s sink and hands it over to libao for
+ * 		playback.
+ * 
+ */
 
 #include "config.h"
 
@@ -36,7 +45,6 @@ THE SOFTWARE.
 #include <libavcodec/avcodec.h>
 #include <libavutil/avutil.h>
 #include <libavfilter/avfilter.h>
-#include <libavfilter/avfiltergraph.h>
 #include <libavfilter/buffersink.h>
 #include <libavfilter/buffersrc.h>
 #ifdef HAVE_LIBAVFILTER_AVCODEC_H
@@ -62,23 +70,59 @@ static void printError (const BarSettings_t * const settings,
 }
 
 /*	global initialization
- *
- *	XXX: in theory we can select the filters/formats we want to support, but
- *	this does not work in practise.
  */
-void BarPlayerInit () {
+void BarPlayerInit (player_t * const p, const BarSettings_t * const settings) {
 	ao_initialize ();
 	av_log_set_level (AV_LOG_FATAL);
+#ifdef HAVE_AV_REGISTER_ALL
 	av_register_all ();
+#endif
+#ifdef HAVE_AVFILTER_REGISTER_ALL
 	avfilter_register_all ();
+#endif
+#ifdef HAVE_AVFORMAT_NETWORK_INIT
 	avformat_network_init ();
+#endif
+
+	pthread_mutex_init (&p->lock, NULL);
+	pthread_cond_init (&p->cond, NULL);
+	pthread_mutex_init (&p->aoplayLock, NULL);
+	pthread_cond_init (&p->aoplayCond, NULL);
+	BarPlayerReset (p);
+	p->settings = settings;
 }
 
-void BarPlayerDestroy () {
+void BarPlayerDestroy (player_t * const p) {
+	pthread_cond_destroy (&p->cond);
+	pthread_mutex_destroy (&p->lock);
+	pthread_cond_destroy (&p->aoplayCond);
+	pthread_mutex_destroy (&p->aoplayLock);
+
+#ifdef HAVE_AVFORMAT_NETWORK_INIT
 	avformat_network_deinit ();
+#endif
 	ao_shutdown ();
 }
 
+void BarPlayerReset (player_t * const p) {
+	p->doQuit = false;
+	p->doPause = false;
+	p->songDuration = 0;
+	p->songPlayed = 0;
+	p->mode = PLAYER_DEAD;
+	p->fvolume = NULL;
+	p->fgraph = NULL;
+	p->fctx = NULL;
+	p->st = NULL;
+	p->cctx = NULL;
+	p->fbufsink = NULL;
+	p->fabuf = NULL;
+	p->streamIdx = -1;
+	p->lastTimestamp = 0;
+	p->interrupted = 0;
+	p->aoDev = NULL;
+}
+
 /*	Update volume filter
  */
 void BarPlayerSetVolume (player_t * const player) {
@@ -122,7 +166,9 @@ static int intCb (void * const data) {
 	assert (player != NULL);
 	if (player->interrupted > 1) {
 		/* got a sigint multiple times, quit pianobar (handled by main.c). */
+		pthread_mutex_lock (&player->lock);
 		player->doQuit = true;
+		pthread_mutex_unlock (&player->lock);
 		return 1;
 	} else if (player->interrupted != 0) {
 		/* the request is retried with the same player context */
@@ -145,8 +191,16 @@ static bool openStream (player_t * const player) {
 	player->fctx->interrupt_callback.callback = intCb;
 	player->fctx->interrupt_callback.opaque = player;
 
+	/* in microseconds */
+	unsigned long int timeout = player->settings->timeout*1000000;
+	char timeoutStr[16];
+	ret = snprintf (timeoutStr, sizeof (timeoutStr), "%lu", timeout);
+	assert (ret < sizeof (timeoutStr));
+	AVDictionary *options = NULL;
+	av_dict_set (&options, "timeout", timeoutStr, 0);
+
 	assert (player->url != NULL);
-	if ((ret = avformat_open_input (&player->fctx, player->url, NULL, NULL)) < 0) {
+	if ((ret = avformat_open_input (&player->fctx, player->url, NULL, &options)) < 0) {
 		softfail ("Unable to open audio file");
 	}
 
@@ -190,9 +244,12 @@ static bool openStream (player_t * const player) {
 		av_seek_frame (player->fctx, player->streamIdx, player->lastTimestamp, 0);
 	}
 
-	player->songPlayed = 0;
-	player->songDuration = av_q2d (player->st->time_base) *
+	const unsigned int songDuration = av_q2d (player->st->time_base) *
 			(double) player->st->duration;
+	pthread_mutex_lock (&player->lock);
+	player->songPlayed = 0;
+	player->songDuration = songDuration;
+	pthread_mutex_unlock (&player->lock);
 
 	return true;
 }
@@ -218,14 +275,14 @@ static bool openFilter (player_t * const player) {
 			av_get_sample_fmt_name (player->cctx->sample_fmt),
 			cp->channel_layout);
 	if ((ret = avfilter_graph_create_filter (&player->fabuf,
-			avfilter_get_by_name ("abuffer"), NULL, strbuf, NULL,
+			avfilter_get_by_name ("abuffer"), "source", strbuf, NULL,
 			player->fgraph)) < 0) {
 		softfail ("create_filter abuffer");
 	}
 
 	/* volume */
 	if ((ret = avfilter_graph_create_filter (&player->fvolume,
-			avfilter_get_by_name ("volume"), NULL, "0dB", NULL,
+			avfilter_get_by_name ("volume"), "volume", "0dB", NULL,
 			player->fgraph)) < 0) {
 		softfail ("create_filter volume");
 	}
@@ -235,14 +292,14 @@ static bool openFilter (player_t * const player) {
 	snprintf (strbuf, sizeof (strbuf), "sample_fmts=%s",
 			av_get_sample_fmt_name (avformat));
 	if ((ret = avfilter_graph_create_filter (&fafmt,
-					avfilter_get_by_name ("aformat"), NULL, strbuf, NULL,
+					avfilter_get_by_name ("aformat"), "format", strbuf, NULL,
 					player->fgraph)) < 0) {
 		softfail ("create_filter aformat");
 	}
 
 	/* abuffersink */
 	if ((ret = avfilter_graph_create_filter (&player->fbufsink,
-			avfilter_get_by_name ("abuffersink"), NULL, NULL, NULL,
+			avfilter_get_by_name ("abuffersink"), "sink", NULL, NULL,
 			player->fgraph)) < 0) {
 		softfail ("create_filter abuffersink");
 	}
@@ -283,10 +340,34 @@ static bool openDevice (player_t * const player) {
 	return true;
 }
 
+/*	Operating on shared variables and must be protected by mutex
+ */
+
+static bool shouldQuit (player_t * const player) {
+	pthread_mutex_lock (&player->lock);
+	const bool ret = player->doQuit;
+	pthread_mutex_unlock (&player->lock);
+	return ret;
+}
+
+static void changeMode (player_t * const player, unsigned int mode) {
+	pthread_mutex_lock (&player->lock);
+	player->mode = mode;
+	pthread_mutex_unlock (&player->lock);
+}
+
+BarPlayerMode BarPlayerGetMode (player_t * const player) {
+	pthread_mutex_lock (&player->lock);
+	const BarPlayerMode ret = player->mode;
+	pthread_mutex_unlock (&player->lock);
+	return ret;
+}
+
 /*	decode and play stream. returns 0 or av error code.
  */
 static int play (player_t * const player) {
 	assert (player != NULL);
+	const int64_t minBufferHealth = player->settings->bufferSecs;
 
 	AVPacket pkt;
 	AVCodecContext * const cctx = player->cctx;
@@ -294,15 +375,15 @@ static int play (player_t * const player) {
 	pkt.data = NULL;
 	pkt.size = 0;
 
-	AVFrame *frame = NULL, *filteredFrame = NULL;
+	AVFrame *frame = NULL;
 	frame = av_frame_alloc ();
 	assert (frame != NULL);
-	filteredFrame = av_frame_alloc ();
-	assert (filteredFrame != NULL);
-
+	pthread_t aoplaythread;
+	pthread_create (&aoplaythread, NULL, BarAoPlayThread, player);
 	enum { FILL, DRAIN, DONE } drainMode = FILL;
 	int ret = 0;
-	while (!player->doQuit && drainMode != DONE) {
+	const double timeBase = av_q2d (player->st->time_base);
+	while (!shouldQuit (player) && drainMode != DONE) {
 		if (drainMode == FILL) {
 			ret = av_read_frame (player->fctx, &pkt);
 			if (ret == AVERROR_EOF) {
@@ -315,6 +396,12 @@ static int play (player_t * const player) {
 				continue;
 			} else if (ret < 0) {
 				/* error, abort */
+				/* mark the EOF, so that BarAoPlayThread can quit*/
+				pthread_mutex_lock (&player->aoplayLock);
+				const int rt = av_buffersrc_add_frame (player->fabuf, NULL);
+				assert (rt == 0);
+				pthread_cond_broadcast (&player->aoplayCond);
+				pthread_mutex_unlock (&player->aoplayLock);
 				break;
 			} else {
 				/* fill buffer */
@@ -322,22 +409,17 @@ static int play (player_t * const player) {
 			}
 		}
 
-		/* pausing */
-		pthread_mutex_lock (&player->pauseMutex);
-		if (player->doPause) {
-			av_read_pause (player->fctx);
-			do {
-				pthread_cond_wait (&player->pauseCond, &player->pauseMutex);
-			} while (player->doPause);
-			av_read_play (player->fctx);
-		}
-		pthread_mutex_unlock (&player->pauseMutex);
-
-		while (!player->doQuit) {
+		while (!shouldQuit (player)) {
 			ret = avcodec_receive_frame (cctx, frame);
 			if (ret == AVERROR_EOF) {
 				/* done draining */
 				drainMode = DONE;
+				/* mark the EOF*/
+				pthread_mutex_lock (&player->aoplayLock);
+				const int rt = av_buffersrc_add_frame (player->fabuf, NULL);
+				assert (rt == 0);
+				pthread_cond_broadcast (&player->aoplayCond);
+				pthread_mutex_unlock (&player->aoplayLock);
 				break;
 			} else if (ret != 0) {
 				/* no more output */
@@ -348,33 +430,29 @@ static int play (player_t * const player) {
 			if (frame->pts == (int64_t) AV_NOPTS_VALUE) {
 				frame->pts = 0;
 			}
+			pthread_mutex_lock (&player->aoplayLock);
 			ret = av_buffersrc_write_frame (player->fabuf, frame);
 			assert (ret >= 0);
-
-			while (true) {
-				if (av_buffersink_get_frame (player->fbufsink, filteredFrame) < 0) {
-					/* try again next frame */
-					break;
+			pthread_mutex_unlock (&player->aoplayLock);
+			
+			int64_t bufferHealth = 0;
+			do {
+				pthread_mutex_lock (&player->aoplayLock);
+				bufferHealth = timeBase * (double) (frame->pts - player->lastTimestamp);
+				if (bufferHealth > minBufferHealth) {
+					/* Buffer get healthy, resume */
+					pthread_cond_broadcast (&player->aoplayCond);
+					/* Buffer is healthy enough, wait */
+					pthread_cond_wait (&player->aoplayCond, &player->aoplayLock);
 				}
-
-				const int numChannels = av_get_channel_layout_nb_channels (
-						filteredFrame->channel_layout);
-				const int bps = av_get_bytes_per_sample(filteredFrame->format);
-				ao_play (player->aoDev, (char *) filteredFrame->data[0],
-						filteredFrame->nb_samples * numChannels * bps);
-
-				av_frame_unref (filteredFrame);
-			}
+				pthread_mutex_unlock (&player->aoplayLock);
+			} while (bufferHealth > minBufferHealth);
 		}
 
-		player->songPlayed = av_q2d (player->st->time_base) * (double) pkt.pts;
-		player->lastTimestamp = pkt.pts;
-
 		av_packet_unref (&pkt);
 	}
-
-	av_frame_free (&filteredFrame);
 	av_frame_free (&frame);
+	pthread_join (aoplaythread, NULL);
 
 	return ret;
 }
@@ -410,7 +488,7 @@ void *BarPlayerThread (void *data) {
 		retry = false;
 		if (openStream (player)) {
 			if (openFilter (player) && openDevice (player)) {
-				player->mode = PLAYER_PLAYING;
+				changeMode (player, PLAYER_PLAYING);
 				BarPlayerSetVolume (player);
 				retry = play (player) == AVERROR_INVALIDDATA &&
 						!player->interrupted;
@@ -422,12 +500,74 @@ void *BarPlayerThread (void *data) {
 			/* stream not found */
 			pret = PLAYER_RET_SOFTFAIL;
 		}
-		player->mode = PLAYER_WAITING;
+		changeMode (player, PLAYER_WAITING);
 		finish (player);
 	} while (retry);
 
-	player->mode = PLAYER_FINISHED;
+	changeMode (player, PLAYER_FINISHED);
 
 	return (void *) pret;
 }
 
+void *BarAoPlayThread (void *data) {
+	assert (data != NULL);
+
+	player_t * const player = data;
+
+	AVFrame *filteredFrame = NULL;
+	filteredFrame = av_frame_alloc ();
+	assert (filteredFrame != NULL);
+
+	int ret;
+	const double timeBase = av_q2d (av_buffersink_get_time_base (player->fbufsink)),
+			timeBaseSt = av_q2d (player->st->time_base);
+	while (!shouldQuit(player)) {
+		pthread_mutex_lock (&player->aoplayLock);
+		ret = av_buffersink_get_frame (player->fbufsink, filteredFrame);
+		if (ret == AVERROR_EOF || shouldQuit (player)) {
+			/* we are done here */
+			pthread_mutex_unlock (&player->aoplayLock);
+			break;
+		} else if (ret < 0) {
+			/* wait for more frames */
+			pthread_cond_broadcast (&player->aoplayCond);
+			pthread_cond_wait (&player->aoplayCond, &player->aoplayLock);
+			pthread_mutex_unlock (&player->aoplayLock);
+			continue;
+		}
+		pthread_mutex_unlock (&player->aoplayLock);
+
+		const int numChannels = av_get_channel_layout_nb_channels (
+				filteredFrame->channel_layout);
+		const int bps = av_get_bytes_per_sample (filteredFrame->format);
+		ao_play (player->aoDev, (char *) filteredFrame->data[0],
+				filteredFrame->nb_samples * numChannels * bps);
+
+		const double timestamp = (double) filteredFrame->pts * timeBase;
+		const unsigned int songPlayed = timestamp;
+
+		pthread_mutex_lock (&player->lock);
+		player->songPlayed = songPlayed;
+		/* pausing */
+		if (player->doPause) {
+			do {
+				pthread_cond_wait (&player->cond, &player->lock);
+			} while (player->doPause);
+		}
+		pthread_mutex_unlock (&player->lock);
+
+		/* lastTimestamp must be the last pts, but expressed in terms of
+		 * st->time_base, not the sink’s time_base. */
+		const int64_t lastTimestamp = timestamp/timeBaseSt;
+		/* notify download thread, we might need more data */
+		pthread_mutex_lock (&player->aoplayLock);
+		player->lastTimestamp = lastTimestamp;
+		pthread_cond_broadcast (&player->aoplayCond);
+		pthread_mutex_unlock (&player->aoplayLock);
+
+		av_frame_unref (filteredFrame);
+	}
+	av_frame_free (&filteredFrame);
+
+	return (void *) 0;
+}
