@@ -84,7 +84,7 @@ typedef struct {
 
 typedef struct {
 	bool autoselect;
-	unsigned int history, maxPlayerErrors;
+	unsigned int history, maxRetry, timeout, bufferSecs;
 	int volume;
 	float gainMul;
 	BarStationSorting_t sortOrder;
@@ -96,8 +96,7 @@ typedef struct {
 	char *bindTo;
 	char *autostartStation;
 	char *eventCmd;
-	char *loveIcon;
-	char *banIcon;
+	char *loveIcon, *banIcon, *tiredIcon;
 	char *atIcon;
 	char *npSongFormat;
 	char *npStationFormat;
