@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2008-2013
+Copyright (c) 2008-2018
 	Lars-Dominik Braun <lars@6xq.net>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -205,6 +205,7 @@ static CURLcode BarPianoHttpRequest (CURL * const http,
 	setAndCheck (CURLOPT_PROGRESSDATA, &lint);
 	setAndCheck (CURLOPT_NOPROGRESS, 0);
 	setAndCheck (CURLOPT_POST, 1);
+	setAndCheck (CURLOPT_TIMEOUT, settings->timeout);
 	if (settings->caBundle != NULL) {
 		setAndCheck (CURLOPT_CAINFO, settings->caBundle);
 	}
@@ -241,7 +242,21 @@ static CURLcode BarPianoHttpRequest (CURL * const http,
 	list = curl_slist_append (list, "Content-Type: text/plain");
 	setAndCheck (CURLOPT_HTTPHEADER, list);
 
-	httpret = curl_easy_perform (http);
+	unsigned int retry = 0;
+	do {
+		httpret = curl_easy_perform (http);
+		++retry;
+		if (httpret == CURLE_OPERATION_TIMEDOUT) {
+			free (buffer.data);
+			buffer.data = NULL;
+			buffer.pos = 0;
+			if (retry >= settings->maxRetry) {
+				break;
+			}
+		} else {
+			break;
+		}
+	} while (true);
 
 	curl_slist_free_all (list);
 
@@ -486,20 +501,21 @@ PianoStation_t *BarUiSelectStation (BarApp_t *app, PianoStation_t *stations,
 }
 
 /*	let user pick one song
- *	@param pianobar settings
+ *	@param app
  *	@param song list
  *	@param input fds
  *	@return pointer to selected item in song list or NULL
  */
-PianoSong_t *BarUiSelectSong (const BarSettings_t *settings,
+PianoSong_t *BarUiSelectSong (const BarApp_t * const app,
 		PianoSong_t *startSong, BarReadlineFds_t *input) {
+	const BarSettings_t * const settings = &app->settings;
 	PianoSong_t *tmpSong = NULL;
 	char buf[100];
 
 	memset (buf, 0, sizeof (buf));
 
 	do {
-		BarUiListSongs (settings, startSong, buf);
+		BarUiListSongs (app, startSong, buf);
 
 		BarUiMsg (settings, MSG_QUESTION, "Select song: ");
 		if (BarReadlineStr (buf, sizeof (buf), input, BAR_RL_DEFAULT) == 0) {
@@ -598,7 +614,7 @@ char *BarUiSelectMusicId (BarApp_t *app, PianoStation_t *station,
 					musicId = strdup (tmpArtist->musicId);
 				}
 			} else if (*selectBuf == 't') {
-				tmpSong = BarUiSelectSong (&app->settings, searchResult.songs,
+				tmpSong = BarUiSelectSong (app, searchResult.songs,
 						&app->input);
 				if (tmpSong != NULL) {
 					musicId = strdup (tmpSong->musicId);
@@ -606,7 +622,7 @@ char *BarUiSelectMusicId (BarApp_t *app, PianoStation_t *station,
 			}
 		} else if (searchResult.songs != NULL) {
 			/* songs found */
-			tmpSong = BarUiSelectSong (&app->settings, searchResult.songs,
+			tmpSong = BarUiSelectSong (app, searchResult.songs,
 					&app->input);
 			if (tmpSong != NULL) {
 				musicId = strdup (tmpSong->musicId);
@@ -714,6 +730,23 @@ void BarUiPrintStation (const BarSettings_t *settings,
 	BarUiMsg (settings, MSG_PLAYING, "%s", outstr);
 }
 
+static const char *ratingToIcon (const BarSettings_t * const settings,
+		const PianoSong_t * const song) {
+	switch (song->rating) {
+		case PIANO_RATE_LOVE:
+			return settings->loveIcon;
+
+		case PIANO_RATE_BAN:
+			return settings->banIcon;
+
+		case PIANO_RATE_TIRED:
+			return settings->tiredIcon;
+
+		default:
+			return "";
+	}
+}
+
 /*	Print song infos (artist, title, album, loved)
  *	@param pianobar settings
  *	@param the song
@@ -723,7 +756,7 @@ void BarUiPrintSong (const BarSettings_t *settings,
 		const PianoSong_t *song, const PianoStation_t *station) {
 	char outstr[512];
 	const char *vals[] = {song->title, song->artist, song->album,
-			(song->rating == PIANO_RATE_LOVE) ? settings->loveIcon : "",
+			ratingToIcon (settings, song),
 			station != NULL ? settings->atIcon : "",
 			station != NULL ? station->name : "",
 			song->detailUrl};
@@ -740,23 +773,44 @@ void BarUiPrintSong (const BarSettings_t *settings,
  *	@param artist/song filter string
  *	@return # of songs
  */
-size_t BarUiListSongs (const BarSettings_t *settings,
+size_t BarUiListSongs (const BarApp_t * const app,
 		const PianoSong_t *song, const char *filter) {
+	const BarSettings_t * const settings = &app->settings;
 	size_t i = 0;
-	char digits[8];
 
 	PianoListForeachP (song) {
 		if (filter == NULL ||
 				(filter != NULL && (BarStrCaseStr (song->artist, filter) != NULL ||
 				BarStrCaseStr (song->title, filter) != NULL))) {
-			char outstr[512];
-			const char *vals[] = {digits, song->artist, song->title,
-					(song->rating == PIANO_RATE_LOVE) ? settings->loveIcon :
-					((song->rating == PIANO_RATE_BAN) ? settings->banIcon : "")};
+			const char * const deleted = "(deleted)", * const empty = "";
+			const char *stationName = empty;
 
+			const PianoStation_t * const station =
+					PianoFindStationById (app->ph.stations, song->stationId);
+			if (station != NULL && station != app->curStation) {
+				stationName = station->name;
+			} else if (station == NULL && song->stationId != NULL) {
+				stationName = deleted;
+			}
+
+			char outstr[512], digits[8], duration[8] = "??:??";
+			const char *vals[] = {digits, song->artist, song->title,
+					ratingToIcon (settings, song),
+					duration,
+					stationName != empty ? settings->atIcon : "",
+					stationName,
+					};
+
+			/* pre-format a few strings */
 			snprintf (digits, sizeof (digits) / sizeof (*digits), "%2zu", i);
+			const unsigned int length = song->length;
+			if (length > 0) {
+				snprintf (duration, sizeof (duration), "%02u:%02u",
+						length / 60, length % 60);
+			}
+
 			BarUiCustomFormat (outstr, sizeof (outstr), settings->listSongFormat,
-					"iatr", vals);
+					"iatrd@s", vals);
 			BarUiAppendNewline (outstr, sizeof (outstr));
 			BarUiMsg (settings, MSG_LIST, "%s", outstr);
 		}
@@ -775,7 +829,7 @@ size_t BarUiListSongs (const BarSettings_t *settings,
  */
 void BarUiStartEventCmd (const BarSettings_t *settings, const char *type,
 		const PianoStation_t *curStation, const PianoSong_t *curSong,
-		const player_t * const player, PianoStation_t *stations,
+		player_t * const player, PianoStation_t *stations,
 		PianoReturn_t pRet, CURLcode wRet) {
 	pid_t chld;
 	int pipeFd[2];
@@ -816,6 +870,11 @@ void BarUiStartEventCmd (const BarSettings_t *settings, const char *type,
 			songStation = PianoFindStationById (stations, curSong->stationId);
 		}
 
+		pthread_mutex_lock (&player->lock);
+		const unsigned int songDuration = player->songDuration;
+		const unsigned int songPlayed = player->songPlayed;
+		pthread_mutex_unlock (&player->lock);
+
 		fprintf (pipeWriteFd,
 				"artist=%s\n"
 				"title=%s\n"
@@ -841,8 +900,8 @@ void BarUiStartEventCmd (const BarSettings_t *settings, const char *type,
 				PianoErrorToStr (pRet),
 				wRet,
 				curl_easy_strerror (wRet),
-				player->songDuration,
-				player->songPlayed,
+				songDuration,
+				songPlayed,
 				curSong == NULL ? PIANO_RATE_NONE : curSong->rating,
 				curSong == NULL ? "" : curSong->detailUrl
 				);
