@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2008-2013
+Copyright (c) 2008-2018
 	Lars-Dominik Braun <lars@6xq.net>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -52,11 +52,14 @@ THE SOFTWARE.
 static inline void BarUiDoSkipSong (player_t * const player) {
 	assert (player != NULL);
 
-	pthread_mutex_lock (&player->pauseMutex);
+	pthread_mutex_lock (&player->lock);
 	player->doQuit = true;
 	player->doPause = false;
-	pthread_cond_broadcast (&player->pauseCond);
-	pthread_mutex_unlock (&player->pauseMutex);
+	pthread_cond_broadcast (&player->cond);
+	pthread_mutex_unlock (&player->lock);
+	pthread_mutex_lock (&player->aoplayLock);
+	pthread_cond_broadcast (&player->aoplayCond);
+	pthread_mutex_unlock (&player->aoplayLock);
 }
 
 /*	transform station if necessary to allow changes like rename, rate, ...
@@ -419,28 +422,28 @@ BarUiActCallback(BarUiActSkipSong) {
 /*	play
  */
 BarUiActCallback(BarUiActPlay) {
-	pthread_mutex_lock (&app->player.pauseMutex);
+	pthread_mutex_lock (&app->player.lock);
 	app->player.doPause = false;
-	pthread_cond_broadcast (&app->player.pauseCond);
-	pthread_mutex_unlock (&app->player.pauseMutex);
+	pthread_cond_broadcast (&app->player.cond);
+	pthread_mutex_unlock (&app->player.lock);
 }
 
 /*	pause
  */
 BarUiActCallback(BarUiActPause) {
-	pthread_mutex_lock (&app->player.pauseMutex);
+	pthread_mutex_lock (&app->player.lock);
 	app->player.doPause = true;
-	pthread_cond_broadcast (&app->player.pauseCond);
-	pthread_mutex_unlock (&app->player.pauseMutex);
+	pthread_cond_broadcast (&app->player.cond);
+	pthread_mutex_unlock (&app->player.lock);
 }
 
 /*	toggle pause
  */
 BarUiActCallback(BarUiActTogglePause) {
-	pthread_mutex_lock (&app->player.pauseMutex);
+	pthread_mutex_lock (&app->player.lock);
 	app->player.doPause = !app->player.doPause;
-	pthread_cond_broadcast (&app->player.pauseCond);
-	pthread_mutex_unlock (&app->player.pauseMutex);
+	pthread_cond_broadcast (&app->player.cond);
+	pthread_mutex_unlock (&app->player.lock);
 }
 
 /*	rename current station
@@ -505,7 +508,7 @@ BarUiActCallback(BarUiActTempBanSong) {
 BarUiActCallback(BarUiActPrintUpcoming) {
 	PianoSong_t * const nextSong = PianoListNextP (selSong);
 	if (nextSong != NULL) {
-		BarUiListSongs (&app->settings, nextSong, NULL);
+		BarUiListSongs (app, nextSong, NULL);
 	} else {
 		BarUiMsg (&app->settings, MSG_INFO, "No songs in queue.\n");
 	}
@@ -561,15 +564,15 @@ BarUiActCallback(BarUiActSelectQuickMix) {
 	if (selStation->isQuickMix) {
 		PianoStation_t *toggleStation;
 		while ((toggleStation = BarUiSelectStation (app, app->ph.stations,
-				"Toggle quickmix for station: ",
+				"Toggle QuickMix for station: ",
 				BarUiActQuickmixCallback, false)) != NULL) {
 			toggleStation->useQuickMix = !toggleStation->useQuickMix;
 		}
-		BarUiMsg (&app->settings, MSG_INFO, "Setting quickmix stations... ");
+		BarUiMsg (&app->settings, MSG_INFO, "Setting QuickMix stations... ");
 		BarUiActDefaultPianoCall (PIANO_REQUEST_SET_QUICKMIX, NULL);
 		BarUiActDefaultEventcmd ("stationquickmixtoggle");
 	} else {
-		BarUiMsg (&app->settings, MSG_ERR, "Not a QuickMix station.\n");
+		BarUiMsg (&app->settings, MSG_ERR, "Please select a QuickMix station first.\n");
 	}
 }
 
@@ -587,7 +590,7 @@ BarUiActCallback(BarUiActHistory) {
 	PianoSong_t *histSong;
 
 	if (app->songHistory != NULL) {
-		histSong = BarUiSelectSong (&app->settings, app->songHistory,
+		histSong = BarUiSelectSong (app, app->songHistory,
 				&app->input);
 		if (histSong != NULL) {
 			BarKeyShortcutId_t action;
@@ -842,7 +845,7 @@ BarUiActCallback(BarUiActManageStation) {
 				BarUiActDefaultEventcmd ("stationdeleteartistseed");
 			}
 		} else if (selectBuf[0] == 's') {
-			PianoSong_t *song = BarUiSelectSong (&app->settings,
+			PianoSong_t *song = BarUiSelectSong (app,
 					reqData.info.songSeeds, &app->input);
 			if (song != NULL) {
 				PianoRequestDataDeleteSeed_t subReqData;
@@ -869,7 +872,7 @@ BarUiActCallback(BarUiActManageStation) {
 				BarUiActDefaultEventcmd ("stationdeletestationseed");
 			}
 		} else if (selectBuf[0] == 'f') {
-			PianoSong_t *song = BarUiSelectSong (&app->settings,
+			PianoSong_t *song = BarUiSelectSong (app,
 					reqData.info.feedback, &app->input);
 			if (song != NULL) {
 				BarUiMsg (&app->settings, MSG_INFO, "Deleting feedback... ");
@@ -881,4 +884,3 @@ BarUiActCallback(BarUiActManageStation) {
 
 	PianoDestroyStationInfo (&reqData.info);
 }
-
